@@ -50,6 +50,12 @@
   #include <regex.h>
 #endif
 
+/* For CommandLineToArgvW. */
+#if defined(__MINGW32__) || defined(_MSC_VER)
+#include <windows.h>
+#include <shellapi.h>
+#endif
+
 /* ppriv - NO_INTERACTIVE_DEPENDENCY - for simpler debugging in Visual Studio
  *
  */
@@ -188,6 +194,14 @@ void setGlobalVerboseLevel(int argc, char**argv)
     }while(pos != string::npos);
   }
 
+  /* print LOG_GBODE if LOG_GBODE_V if active */
+  if(useStream[LOG_GBODE_V] == 1)
+    useStream[LOG_GBODE] = 1;
+
+  /* print LOG_GBODE_NLS if LOG_GBODE_NLS_V if active */
+  if(useStream[LOG_GBODE_NLS_V] == 1)
+    useStream[LOG_GBODE_NLS] = 1;
+
   /* print LOG_INIT and LOG_SOTI if LOG_INIT_V is active */
   if(useStream[LOG_INIT_V] == 1)
   {
@@ -421,6 +435,7 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
 
   /* linear model option is set : <-l lintime> */
   int create_linearmodel = omc_flag[FLAG_L];
+  data->modelData->create_linearmodel = create_linearmodel;
   const char* lintime = omc_flagValue[FLAG_L];
 
   /* activated measure time option with LOG_STATS */
@@ -531,6 +546,8 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
     data->modelData->resultFileName = GC_strdup(result_file_cstr.c_str());
   }
 
+  data->modelData->resourcesDir = NULL;
+
   string init_initMethod = "";
   string init_file = "";
   string init_time_string = "";
@@ -542,11 +559,10 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
     init_initMethod = omc_flagValue[FLAG_IIM];
   }
   if(omc_flag[FLAG_IIF]) {
-    omc_stat_t attrib;
     if (omc_flag[FLAG_INPUT_PATH]) {
       const char *tmp_filename;
 
-      if (omc_stat(omc_flagValue[FLAG_IIF], &attrib ) == 0) {
+      if (omc_file_exists(omc_flagValue[FLAG_IIF])) {
         if (0 > GC_asprintf(&tmp_filename, "%s", omc_flagValue[FLAG_IIF] )) {
           throwStreamPrint(NULL, "simulation_runtime.cpp: Error: can not allocate memory.");
         }
@@ -561,7 +577,7 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
     else {
       init_file = omc_flagValue[FLAG_IIF];
     }
-    if (omc_stat(init_file.c_str(), &attrib ) != 0) {
+    if (!omc_file_exists(init_file.c_str())) {
       throwStreamPrint(NULL, "Initialization file \"%s\" doesn't exist.", init_file.c_str());
     }
   }
@@ -619,10 +635,18 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
   {
     infoStreamPrint(LOG_STDOUT, 0, "Reconcile Boundary Conditions Starting!");
     infoStreamPrint(LOG_STDOUT, 0, "%s", data->modelData->modelName);
-    retVal = reconcileBoundaryConditions(data, threadData, retVal);
+    retVal = boundaryConditions(data, threadData, retVal);
     infoStreamPrint(LOG_STDOUT, 0, "Reconcile Boundary Conditions Completed!");
   }
 
+  if (omc_flag[FLAG_DATA_RECONCILE_STATE])
+  {
+    infoStreamPrint(LOG_STDOUT, 0, "Reconcile State Estimation Starting!");
+    infoStreamPrint(LOG_STDOUT, 0, "%s", data->modelData->modelName);
+    retVal = dataReconciliation(data, threadData, retVal);
+    infoStreamPrint(LOG_STDOUT, 0, "Reconcile State Estimation Completed!");
+  }
+
   if(0 == retVal && create_linearmodel) {
     rt_tick(SIM_TIMER_JACOBIAN);
     retVal = linearize(data, threadData);
@@ -1143,18 +1167,22 @@ void communicateMsg(char id, unsigned int size, const char *data)
 #endif
 }
 
-
-/* \brief main function for simulator
+/**
+ * @brief Parses the commandline (program options) and sets some
+ * values. See initRuntimeAndSimulation for more info.
+ * This allows generated simulation code to check-on/read options and flags before
+ * it calls the main _main_SimulationRuntime function to do the simulation.
  *
- * The arguments for the main function are:
- * -v verbose = debug
- * -vf = flags set verbosity flags
- * -f init_file.txt use input data from init file.
- * -r res.plt write result to file.
+ * @param argc
+ * @param argv  This gets overwritten on Windows!!
+ * @param data
+ * @param threadData
+ * @return int    Returns 0 on success. Returns 1 otherwise.
+ *
+ * Note: The function will overwrite argv to its wide character representation. Not sure
+ * if this is a good idea. However, I am leaving it as it was for now.
  */
-
-int _main_SimulationRuntime(int argc, char**argv, DATA *data, threadData_t *threadData)
-{
+int _main_initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *threadData) {
 #if defined(__MINGW32__) || defined(_MSC_VER)
   /* Support for non-ASCII characters
    * Read the unicode command line arguments and replace the normal arguments with it.
@@ -1167,11 +1195,26 @@ int _main_SimulationRuntime(int argc, char**argv, DATA *data, threadData_t *thre
   }
 #endif
 
-  int retVal = -1;
-  MMC_TRY_INTERNAL(globalJumpBuffer)
   if (initRuntimeAndSimulation(argc, argv, data, threadData)) //initRuntimeAndSimulation returns 1 if an error occurs
     return 1;
 
+  return 0;
+}
+
+/* \brief main function for simulator
+ *
+ * The arguments for the main function are:
+ * -v verbose = debug
+ * -vf = flags set verbosity flags
+ * -f init_file.txt use input data from init file.
+ * -r res.plt write result to file.
+ */
+
+int _main_SimulationRuntime(int argc, char**argv, DATA *data, threadData_t *threadData)
+{
+  int retVal = -1;
+  MMC_TRY_INTERNAL(globalJumpBuffer)
+
   /* sighandler_t oldhandler = different type on all platforms... */
 #ifdef SIGUSR1
   SimulationRuntime_printStatus_data = data; /* Global, but at least we get something back; doesn't matter which simulation run */
@@ -1248,7 +1291,7 @@ static inline void sendXMLTCPIfClosed()
   }
 }
 
-static void messageXMLTCP(int type, int stream, int indentNext, char *msg, int subline, const int *indexes)
+static void messageXMLTCP(int type, int stream, FILE_INFO info, int indentNext, char *msg, int subline, const int *indexes)
 {
   numOpenTags++;
   xmlTcpStream << "<message stream=\"" << LOG_STREAM_NAME[stream] << "\" type=\"" << LOG_TYPE_DESC[type] << "\" text=\"";
@@ -1305,7 +1348,7 @@ static void printEscapedXML(const char *msg)
   }
 }
 
-static void messageXML(int type, int stream, int indentNext, char *msg, int subline, const int *indexes)
+static void messageXML(int type, int stream, FILE_INFO info, int indentNext, char *msg, int subline, const int *indexes)
 {
   printf("<message stream=\"%s\" type=\"%s\" text=\"", LOG_STREAM_NAME[stream], LOG_TYPE_DESC[type]);
   printEscapedXML(msg);
