@@ -115,7 +115,7 @@ public:
   QString getErrorLevel();
   int getErrorId();
   QString getVersion(QString className = QString("OpenModelica"));
-  void loadSystemLibraries();
+  void loadSystemLibraries(const QVector<QPair<QString, QString> > libraries);
   void loadUserLibraries();
   QStringList getClassNames(QString className = QString("AllLoadedClasses"), bool recursive = false, bool qualified = false,
                             bool sort = false, bool builtin = false, bool showProtected = true, bool includeConstants = false);
@@ -129,12 +129,13 @@ public:
   bool isPartial(QString className);
   bool isReplaceable(QString parentClassName, QString className);
   StringHandler::ModelicaClasses getClassRestriction(QString className);
+  bool setParameterValue(const QString &className, const QString &parameter, const QString &value);
   QString getParameterValue(const QString &className, const QString &parameter);
-  QStringList getComponentModifierNames(QString className, QString name);
-  QString getComponentModifierValue(QString className, QString name);
-  bool setComponentModifierValue(QString className, QString name, QString modifierValue);
-  bool removeComponentModifiers(QString className, QString name);
-  QString getComponentModifierValues(QString className, QString name);
+  QStringList getElementModifierNames(QString className, QString name);
+  QString getElementModifierValue(QString className, QString name);
+  bool setElementModifierValue(QString className, QString name, QString modifierValue);
+  bool removeElementModifiers(QString className, QString name);
+  QString getElementModifierValues(QString className, QString name);
   QStringList getExtendsModifierNames(QString className, QString extendsClassName);
   QString getExtendsModifierValue(QString className, QString extendsClassName, QString modifierName);
   bool setExtendsModifierValue(QString className, QString extendsClassName, QString modifierName, QString modifierValue);
@@ -152,9 +153,7 @@ public:
   QList<QString> getInheritedClasses(QString className);
   QString getNthInheritedClassIconMapAnnotation(QString className, int num);
   QString getNthInheritedClassDiagramMapAnnotation(QString className, int num);
-  QList<ElementInfo*> getComponents(QString className);
   QList<ElementInfo*> getElements(QString className);
-  QStringList getComponentAnnotations(QString className);
   QStringList getElementAnnotations(QString className);
   QString getDocumentationAnnotationInfoHeader(LibraryTreeItem *pLibraryTreeItem, QString infoHeader);
   QString getDocumentationAnnotation(LibraryTreeItem *pLibraryTreeItem);
@@ -196,13 +195,10 @@ public:
   bool setComponentDimensions(QString className, QString componentName, QString dimensions);
   bool addConnection(QString from, QString to, QString className, QString annotation);
   bool deleteConnection(QString from, QString to, QString className);
-  bool addTransition(QString className, QString from, QString to, QString condition, bool immediate, bool reset, bool synchronize,
-                     int priority, QString annotation);
-  bool deleteTransition(QString className, QString from, QString to, QString condition, bool immediate, bool reset, bool synchronize,
-                        int priority);
-  bool updateTransition(QString className, QString from, QString to, QString oldCondition, bool oldImmediate, bool oldReset,
-                        bool oldSynchronize, int oldPriority, QString condition, bool immediate, bool reset, bool synchronize, int priority,
-                        QString annotation);
+  bool addTransition(QString className, QString from, QString to, QString condition, bool immediate, bool reset, bool synchronize, int priority, QString annotation);
+  bool deleteTransition(QString className, QString from, QString to, QString condition, bool immediate, bool reset, bool synchronize, int priority);
+  bool updateTransition(QString className, QString from, QString to, QString oldCondition, bool oldImmediate, bool oldReset, bool oldSynchronize, int oldPriority,
+                        QString condition, bool immediate, bool reset, bool synchronize, int priority, QString annotation);
   bool addInitialState(QString className, QString state, QString annotation);
   bool deleteInitialState(QString className, QString state);
   bool updateInitialState(QString className, QString state, QString annotation);
@@ -219,7 +215,7 @@ public:
   OMCInterface::getSimulationOptions_res getSimulationOptions(QString className, double defaultTolerance = 1e-6);
   QString buildModelFMU(QString className, QString version, QString type, QString fileNamePrefix, QList<QString> platforms, bool includeResources);
   QString translateModelXML(QString className);
-  QString importFMU(QString fmuName, QString outputDirectory, int logLevel, bool debugLogging, bool generateInputConnectors, bool generateOutputConnectors);
+  QString importFMU(QString fmuName, QString outputDirectory, int logLevel, bool debugLogging, bool generateInputConnectors, bool generateOutputConnectors, QString modelName);
   QString importFMUModelDescription(QString fmuModelDescriptionName, QString outputDirectory, int logLevel, bool debugLogging, bool generateInputConnectors, bool generateOutputConnectors);
   QString getMatchingAlgorithm();
   OMCInterface::getAvailableMatchingAlgorithms_res getAvailableMatchingAlgorithms();
@@ -282,6 +278,11 @@ public:
   QStringList getAvailablePackageVersions(QString pkg, QString version);
   bool convertPackageToLibrary(const QString &packageToConvert, const QString &library, const QString &libraryVersion);
   QList<QString> getAvailablePackageConversionsFrom(const QString &pkg, const QString &version);
+  QJsonObject getModelInstance(const QString &className, const QString &modifier = QString(""), bool prettyPrint = false, bool icon = false);
+  QJsonObject modifierToJSON(const QString &modifier, bool prettyPrint = false);
+  int storeAST();
+  bool restoreAST(int id);
+  bool clear();
 signals:
   void commandFinished();
 public slots:
