@@ -53,13 +53,14 @@ namespace oms
   public:
     ~ComponentFMUME();
 
-    static Component* NewComponent(const oms::ComRef& cref, System* parentSystem, const std::string& fmuPath);
-    static Component* NewComponent(const pugi::xml_node& node, System* parentSystem,  const std::string& sspVersion, const Snapshot& snapshot);
+    static Component* NewComponent(const oms::ComRef& cref, System* parentSystem, const std::string& fmuPath, std::string replaceComponent = "");
+    static Component* NewComponent(const pugi::xml_node& node, System* parentSystem,  const std::string& sspVersion, const Snapshot& snapshot, std::string variantName);
     const FMUInfo* getFMUInfo() const {return &(this->fmuInfo);}
 
-    oms_status_enu_t exportToSSD(pugi::xml_node& node, Snapshot& snapshot) const;
+    oms_status_enu_t exportToSSD(pugi::xml_node& node, Snapshot& snapshot, std::string variantName) const;
     oms_status_enu_t exportToSSV(pugi::xml_node& ssvNode);
-    oms_status_enu_t exportToSSVTemplate(pugi::xml_node& ssvNode);
+    void getFilteredUnitDefinitionsToSSD(std::map<std::string, std::map<std::string, std::string>>& unitDefinitions);
+    oms_status_enu_t exportToSSVTemplate(pugi::xml_node& ssvNode, Snapshot& snapshot);
     oms_status_enu_t exportToSSMTemplate(pugi::xml_node& ssmNode);
     oms_status_enu_t instantiate();
     oms_status_enu_t initialize();
@@ -77,11 +78,21 @@ namespace oms
     oms_status_enu_t getInteger(const fmi2_value_reference_t& vr, int& value);
     oms_status_enu_t getReal(const ComRef& cref, double& value);
     oms_status_enu_t getReal(const fmi2_value_reference_t& vr, double& value);
+    oms_status_enu_t getString(const ComRef& cref, std::string& value);
+    oms_status_enu_t getString(const fmi2_value_reference_t& vr, std::string& value);
     oms_status_enu_t setBoolean(const ComRef& cref, bool value);
     oms_status_enu_t setInteger(const ComRef& cref, int value);
     oms_status_enu_t setReal(const ComRef& cref, double value);
+    oms_status_enu_t setString(const ComRef& cref, const std::string& value);
+    oms_status_enu_t setUnit(const ComRef& cref, const std::string& value);
+
+    oms_status_enu_t getDirectionalDerivative(const ComRef& unknownCref, const ComRef& knownCref, double& value);
+    oms_status_enu_t getDirectionalDerivativeHeper(const int unknownIndex, const int knownIndex, const std::vector<int>& dependencyList, double& value);
 
     oms_status_enu_t deleteStartValue(const ComRef& cref);
+    oms_status_enu_t updateOrDeleteStartValueInReplacedComponent(std::vector<std::string>& warningList);
+    oms_status_enu_t setValuesResources(std::vector<Values>& allValuesResources);
+    std::vector<Values> getValuesResources();
 
     oms_status_enu_t registerSignalsForResultFile(ResultWriter& resultFile);
     oms_status_enu_t updateSignals(ResultWriter& resultWriter);
@@ -90,7 +101,7 @@ namespace oms
 
     oms_status_enu_t doEventIteration();
 
-    size_t getNumberOfContinuousStates() const {return nContinuousStates;}
+    size_t getNumberOfContinuousStates() const {return derivatives.size();}
     size_t getNumberOfEventIndicators() const {return nEventIndicators;}
     oms_status_enu_t getContinuousStates(double* states);
     oms_status_enu_t setContinuousStates(double* states);
@@ -107,10 +118,13 @@ namespace oms
 
     void getFilteredSignals(std::vector<Connector>& filteredSignals) const;
 
-    oms_status_enu_t addResources(std::string& filename);
+    oms_status_enu_t newResources(const std::string& ssvFilename, const std::string& ssmFilename, bool externalResources);
     oms_status_enu_t setResourcesHelper1(Values value);
     oms_status_enu_t setResourcesHelper2(Values value);
 
+    oms_status_enu_t deleteReferencesInSSD(const std::string& filename);
+    oms_status_enu_t deleteResourcesInSSP(const std::string& filename);
+
   protected:
     ComponentFMUME(const ComRef& cref, System* parentSystem, const std::string& fmuPath);
 
@@ -120,6 +134,8 @@ namespace oms
 
     oms_status_enu_t renameValues(const ComRef& oldCref, const ComRef& newCref);
 
+    void dumpInitialUnknowns();
+
   private:
     jm_callbacks callbacks;
     fmi2_callback_functions_t callbackFunctions;
@@ -127,16 +143,16 @@ namespace oms
     fmi2_import_t* fmu = NULL;
 
     fmi2_event_info_t eventInfo;
-    size_t nContinuousStates;
     size_t nEventIndicators;
 
     FMUInfo fmuInfo;
 
     std::vector<Variable> allVariables;
-    std::vector<Variable> inputs;
-    std::vector<Variable> outputs;
-    std::vector<Variable> parameters;
-    std::vector<Variable> calculatedParameters;
+    std::vector<unsigned int> calculatedParameters;
+    std::vector<unsigned int> derivatives;
+    std::vector<unsigned int> inputs;
+    std::vector<unsigned int> outputs;
+    std::vector<unsigned int> parameters;
     std::vector<bool> exportVariables;
 
     Values values; ///< start values defined before instantiating the FMU and external inputs defined after initialization
